#!/usr/bin/perl -w

## Converts from any version of questionbank to any other programmatically.
## Runs as a filter.
## Christopher Manning 2010.

if ($#ARGV >= 0) {
  $what = $ARGV[0];
} else {
  # $what = 1; # To map from 4000qs-original.txt to 4000qs-20100625.txt
  # $what = 2|4|8|4096; # To map from 4000qs-20100625.txt to qtreebank-4000qs.txt 
  # $what = 1|2|4|8|4096; # To map from 4000qs-original.txt to qtreebank-4000qs.txt 
  # $what = 16|32; # To map from qtreebank-4000qs.txt to qtreebank-4000qs-corrected.txt, sensibly, without one introduced error
  # $what = 16|32|64; # To map from qtreebank-4000qs.txt to qtreebank-4000qs-corrected.txt, exactly
  # $what = 1|2|16|128; # To map 4000qs-original.txt to my old 4000qs-20100625-autofixed.txt
  # $what = 1|2|4|8|16|32|4096; # To map from 4000qs-original.txt to qtreebank-4000qs-corrected.txt, sensibly, without one introduced error
  # $what = 1|2|4|8|16|32|64|4096; # To map from 4000qs-original.txt to qtreebank-4000qs-corrected.txt, exactly
  # $what = 1|2|16|128|256; # To map from 4000qs-original.txt to 4000qs-20100625-withbracket.txt
  # $what = 1|2|16|32|128|256|512|1024; # To map exactly from 4000qs-original.txt to 4000qs.cleaned.txt 
  # $what = 1|2|8|16|32|128|256|512|1024|2048; # Moves towards 3924qs.mrg. It fails to fix the tokenization problem with contractions can't and shan't. You can get exactly to 3924qs.mrg by then using 8192.tsurgeon. This command deletes the 76 test examples.
  $what = 1|2|4|8|16|32|128|256|512|1024|4096; # Current best map from 4000qs-original.txt in line-based mode. Changes after this are done by Tsurgeon.  Use: 8192|16384.  This version keeps all 4000 examples.
  # $what = 1|2|4|8|16|32|128|256|512|1024|2048|4096; # Current best map from 4000qs-original.txt in line-based mode. Changes after this are done by Tsurgeon.  Use: 8192|16384.  This command deletes the 76 test examples.
}

# 1 = Fix WH nonterminals
# 2 = Fix < to ' and parentheses
# 4 = Fix can't tokenization
# 8 = Fix spacing after "What" fix in 1
# 16 = Fix use of ? as POS tag to .
# 32 = Fix use of ' ` as POS tag, and tagging spider as NP
# 64 = ERRONEOUS!  Delete NP node in "in Germany" PP.  Done in qtreebank-4000qs-correct.txt, but a mistake.
# 128 = Fix use of ! as a POS tag to .
# 256 = Put extra set of brackets around all lines (like PTB WSJ)
# 512 = Remove pairs of empty brackets ( )
# 1024 = Correct NPP node to NP
# 2048 = Delete the 76 sentences used in test set of Rimmel et al.
# 4096 = Fix shan't tokenization

# Outside this file
# 8192 = Tsurgeon map to exactly (modulo whitespace) 3924qs.mrg version from Nivre etc.
# 16384 = Fixes that Chris Manning had done previously by hand on 3924qs.mrg and some subsequent fixes generalizing those fixes to other examples.  Gives 3924Stanford.mrg
# 32768 = More fixes


$lineNo=0;
while (<STDIN>) {
  $lineNo++;
  if ($what & 1) {
    # These lines will map from 4000qs-original.txt to 4000qs-20100625.txt
    s/\(WPWhat/\(WP What/g;
    s/\(WHDJP/\(WHADJP/g;
    s/\(WHADJ /\(WHADJP /g;
  }
  if ($what & 2) {
    # These lines will map from 4000qs-20100625.txt to qtreebank-4000qs.txt 
    s/</'/g;
    s/\( (-[LR]RB-)\)/\($1 $1\)/g;
  }
  if ($what & 4) {
    s/can\)\(ADVP \(RB 't\)\)/ca\)\(ADVP \(RB n't\)\)/g;
  }
  if ($what & 4096) {
    s/shan\)\(ADVP \(RB 't\)\)/sha\)\(ADVP \(RB n't\)\)/g;
  }
  if ($what & 8) {
    s/\(WP What \)\)/\(WP What\)\)/g;
  }
  if ($what & 16) {
    # These lines will map from qtreebank-4000qs.txt to qtreebank-4000qs-corrected.txt
    s/\([?] ([!?])\)/(. $1)/g;
  }
  if ($what & 32) {
    s/\(` `\)/\(`` `\)/g;
    s/\(' '\)/\('' '\)/g;
    s/\(NP \(NP spider\)/\(NP \(NN spider\)/g;
  }
  if ($what & 64) {
    # This last fix to line 33 makes things identical to qtreebank-4000qs-corrected.txt
    # but it is introducing a mistake, so we leave it out...
    s/\(NN city\)\)\(PP \(IN in\)\(NP \(NNP Germany\)\)\)\)\)/\(NN city\)\)\(PP \(IN in\) \(NNP Germany\)\)\)\)/g;
  }
  if ($what & 128) {
    s/\([!] ([!?])\)/(. $1)/g;
  }
  if ($what & 256) {
    s/^(.*)$/\( $1 )/;
  }
  if ($what & 512) {
    s/ \( \)/ /g;
  }
  if ($what & 1024) {
    s/\(NPP \(NNPS /\(NP \(NNPS /;
  }
  # 2048 is used below and 4096 is used above


  if ($what & 2048) {
    # This deletes the 76 lines used in Rimmel et al. test data.  Leads to 3924qs.
    if (
        $lineNo == 238 || $lineNo == 257 || $lineNo == 277 || $lineNo == 326 ||
        $lineNo == 342 || $lineNo == 450 || $lineNo == 493 || $lineNo == 500 ||
        $lineNo == 530 || $lineNo == 562 || $lineNo == 581 ||
        $lineNo == 589 || $lineNo == 600 || $lineNo == 604 || $lineNo == 607 || 
        $lineNo == 626 || $lineNo == 628 || $lineNo == 671 || $lineNo == 673 ||
        $lineNo == 675 || $lineNo == 683 || $lineNo == 712 || $lineNo == 731 || $lineNo == 747 ||
        $lineNo == 773 || $lineNo == 781 || $lineNo == 844 || $lineNo == 932 ||
        $lineNo == 933 || $lineNo == 939 || $lineNo == 983 || 
        $lineNo == 1026 || $lineNo == 1037 || $lineNo == 1049 ||
        $lineNo == 1093 || $lineNo == 1102 || $lineNo == 1121 || $lineNo == 1146 ||
        $lineNo == 1151 || $lineNo == 1169 || $lineNo == 1187 || $lineNo == 1206 ||
        $lineNo == 1235 || $lineNo == 1260 || $lineNo == 1321 || $lineNo == 1404 ||
        $lineNo == 1449 || $lineNo == 1468 || $lineNo == 1489 || $lineNo == 1496 ||
        $lineNo == 1522 || $lineNo == 1552 || $lineNo == 1556 || $lineNo == 1583 ||
        $lineNo == 1587 || $lineNo == 1604 || $lineNo == 1642 || $lineNo == 1647 ||
        $lineNo == 1657 || $lineNo == 1661 || $lineNo == 1663 || $lineNo == 1699 ||
        $lineNo == 1703 || $lineNo == 1715 || $lineNo == 1735 || $lineNo == 1738 ||
        $lineNo == 1764 || $lineNo == 1775 || $lineNo == 1824 || $lineNo == 1830 ||
        $lineNo == 1925 || $lineNo == 1947 || $lineNo == 1961 || $lineNo == 1968 ||
        $lineNo == 1971 || $lineNo == 2403   # || $lineNo ==  || $lineNo ==  ||
       ) {
    } else {
      print;
    }
  } else {
    print;
  }
}
